"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _typeof2 = _interopRequireDefault(require("@babel/runtime/helpers/typeof"));

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _fs = _interopRequireDefault(require("fs"));

var _async = _interopRequireDefault(require("async"));

var _xml2js = _interopRequireDefault(require("xml2js"));

var _PipParser = _interopRequireDefault(require("./PipParser"));

function _createForOfIteratorHelper(o, allowArrayLike) { var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"]; if (!it) { if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e) { throw _e; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var normalCompletion = true, didErr = false, err; return { s: function s() { it = it.call(o); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e2) { didErr = true; err = _e2; }, f: function f() { try { if (!normalCompletion && it["return"] != null) it["return"](); } finally { if (didErr) throw err; } } }; }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

var PackageXmlParser = /*#__PURE__*/function () {
  function PackageXmlParser() {
    (0, _classCallCheck2["default"])(this, PackageXmlParser);
  }

  (0, _createClass2["default"])(PackageXmlParser, [{
    key: "parse",
    value: function parse(runner, done) {
      var _this = this;

      var that = this;
      this.pips = runner.options.pips;

      _async["default"].series([function (cb) {
        return _this.readXml('package.xml', cb);
      }, function (cb) {
        return _this.parseInfo(cb);
      }, function (cb) {
        return _this.parsePips(cb);
      }, function (cb) {
        return _this.parseAdditionalPackages(cb);
      }], function (err) {
        if (!err) {
          runner.filesToPackage = that.filesToPackage;
          runner.xmlInfo = that.info;
        }

        done(err);
      });
    }
  }, {
    key: "readXml",
    value: function readXml(file, callback) {
      var that = this;

      _fs["default"].readFile(file, function (err, data) {
        var parser = new _xml2js["default"].Parser();

        if (err) {
          return callback(new Error("The package.xml could not be read."));
        }

        parser.parseString(data, function (err, result) {
          if (err) {
            return callback(new Error("The package.xml does not appear to be a valid XML document."));
          }

          that.xml = result;
          callback();
        });
      });
    }
  }, {
    key: "parseInfo",
    value: function parseInfo(callback) {
      var pack = this.xml["package"];
      var info = {
        name: pack['$'].name,
        author: pack.authorinformation[0].author[0],
        version: pack.packageinformation[0].version[0]
      };
      this.info = info;
      callback();
    }
  }, {
    key: "parsePips",
    value: function parsePips(callback) {
      var instructions = [];
      this.xml["package"].instructions.forEach(function (element) {
        var instruction = element.instruction;
        if (element["void"]) return;
        instruction.forEach(function (element) {
          instructions.push({
            type: element.$.type,
            path: element._
          });
        });
      });
      var list;

      try {
        var parser = new _PipParser["default"](this.pips);
        list = parser.run(instructions);
      } catch (err) {
        callback(err);
        return;
      }

      this.filesToPackage = list.files.map(function (it) {
        return {
          path: it,
          intermediate: false
        };
      });

      if (list && list.styles && list.styles[0]) {
        this.parseStyleXML(list.styles[0], callback);
      } else {
        callback();
      }
    }
  }, {
    key: "parseStyleXML",
    value: function parseStyleXML(path, callback) {
      var that = this;

      _fs["default"].readFile(path + '/style.xml', function (err, data) {
        var parser = new _xml2js["default"].Parser();

        if (err) {
          return callback(new Error("The style.xml could not be read."));
        }

        parser.parseString(data, function (err, result) {
          if (err) {
            return callback(new Error("The style.xml does not appear to be a valid XML document."));
          }

          var additionalPackages = [];

          if (result.style.files) {
            // Only one files section should be parsed
            for (var file in result.style.files[0]) {
              if (file == 'templates' || file == 'images') {
                var tag = result.style.files[0][file][0];
                var filename = (0, _typeof2["default"])(tag) === 'object' ? tag._ : tag;
                additionalPackages.push({
                  path: path + '/' + filename,
                  intermediate: true
                });
              }
            }
          }

          that.filesToPackage = that.filesToPackage.concat(additionalPackages);
          callback();
        });
      });
    }
  }, {
    key: "parseAdditionalPackages",
    value: function parseAdditionalPackages(callback) {
      var packages = [];
      var optionals = this.xml["package"].optionalpackages;
      var requireds = this.xml["package"].requiredpackages;

      var addPackagePaths = function addPackagePaths(paths) {
        var _iterator = _createForOfIteratorHelper(paths),
            _step;

        try {
          for (_iterator.s(); !(_step = _iterator.n()).done;) {
            var path = _step.value;
            var query = [];
            if (path.requiredpackage) query = path.requiredpackage;
            if (path.optionalpackage) query = path.optionalpackage;

            var _iterator2 = _createForOfIteratorHelper(query),
                _step2;

            try {
              for (_iterator2.s(); !(_step2 = _iterator2.n()).done;) {
                var pack = _step2.value;

                if (pack.$.file) {
                  packages.push({
                    path: pack.$.file,
                    intermediate: false,
                    isPackage: true,
                    identifier: pack._
                  });
                }
              }
            } catch (err) {
              _iterator2.e(err);
            } finally {
              _iterator2.f();
            }
          }
        } catch (err) {
          _iterator.e(err);
        } finally {
          _iterator.f();
        }
      };

      if (optionals) addPackagePaths(optionals);
      if (requireds) addPackagePaths(requireds);
      this.filesToPackage = this.filesToPackage.concat(packages);
      callback();
    }
  }]);
  return PackageXmlParser;
}();

exports["default"] = PackageXmlParser;