"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _TreeBuilder = require("./TreeBuilder");

var _PackageXmlParser = _interopRequireDefault(require("./PackageXmlParser"));

var _TaskRunner = _interopRequireDefault(require("./TaskRunner"));

var _Util = _interopRequireDefault(require("./Util"));

var _chalk = _interopRequireDefault(require("chalk"));

var _async = _interopRequireDefault(require("async"));

var _glob = _interopRequireDefault(require("glob"));

var _path = _interopRequireDefault(require("path"));

var _lodash = _interopRequireDefault(require("lodash.uniqby"));

var _tar = _interopRequireDefault(require("tar"));

var _fs = _interopRequireDefault(require("fs"));

var Packager = /*#__PURE__*/function () {
  function Packager(files, packageInfo) {
    (0, _classCallCheck2["default"])(this, Packager);
    this.packageInfo = packageInfo; // Order by intermediate files

    files.sort(function (a, b) {
      if (a.intermediate > b.intermediate) return -1;else return 1;
    }); // Remove duplicates from array

    this.filesToPackage = (0, _lodash["default"])(files, 'path');
  }

  (0, _createClass2["default"])(Packager, [{
    key: "run",
    value: function run(done, destination, quiet) {
      var _this = this;

      this.destination = destination;

      _async["default"].series({
        localFiles: function localFiles(cb) {
          return _this.findLocalFiles(cb);
        },
        treeStructure: function treeStructure(cb) {
          return _this.writeTreeStructure(quiet, cb);
        },
        prepackage: function prepackage(cb) {
          return _this.prepackage(cb);
        },
        "package": function _package(cb) {
          return _this.packageAll(cb);
        },
        fileStats: function fileStats(cb) {
          return _this.getFileStats(cb);
        }
      }, function (err, results) {
        // always run cleanup
        _this.cleanup(function () {
          if (err) {
            done(err);
            return;
          }

          results = {
            filename: results["package"] === undefined ? results["package"] : _path["default"].basename(results["package"]),
            path: results["package"],
            filesize: results.fileStats
          };

          if (!quiet) {
            console.log('-> ' + _chalk["default"].green.bold('Package generated') + ' (' + results.filesize + ')');
          }

          done(err, results);
        });
      });
    }
  }, {
    key: "getFileProcessingList",
    value: function getFileProcessingList() {
      return this.filesToPackage.map(function (item) {
        return function (callback) {
          var path = item.path;
          var adjustedPath = path.replace(/\.tar(\.gz)?$/i, '');
          var newItem = {
            original: path,
            isPackage: item.isPackage,
            identifier: item.identifier,
            originalExists: false
          };

          if (_glob["default"].hasMagic(adjustedPath)) {
            (0, _glob["default"])(adjustedPath, function (err, files) {
              newItem.paths = files;
              callback(err, newItem);
            });
          } else {
            _fs["default"].stat(path, function (err, stats) {
              if (err) {
                newItem.paths = [adjustedPath];
              } else {
                if (item.isPackage) {
                  // add package as tarball and not directory
                  newItem.paths = [path];
                } else {
                  newItem.paths = [adjustedPath];
                }

                newItem.originalExists = true;
              }

              callback(null, newItem);
            });
          }
        };
      });
    }
  }, {
    key: "getFileStats",
    value: function getFileStats(done) {
      _fs["default"].stat(this.getDestinationPath(), function (err, stats) {
        function bytesToSize(bytes) {
          var sizes = ['Bytes', 'KB', 'MB', 'GB'];
          if (bytes == 0) return '0 Byte';
          var i = parseInt(Math.floor(Math.log(bytes) / Math.log(1024)));
          return Math.round(bytes / Math.pow(1000, i), 2) + ' ' + sizes[i];
        }

        ;
        done(err, bytesToSize(stats.size));
      });
    }
  }, {
    key: "findLocalFiles",
    value: function findLocalFiles(done) {
      var that = this;
      var fileProcessingList = this.getFileProcessingList();

      _async["default"].parallel(fileProcessingList, function (err, results) {
        var prepack = [];
        var direct = ['package.xml'];
        results.forEach(function (instruction) {
          if (instruction.isPackage && !instruction.originalExists || !instruction.isPackage && _Util["default"].isTarball(instruction.original)) {
            prepack = prepack.concat(instruction);
          } else {
            direct = direct.concat(instruction.paths.map(function (i) {
              return i.replace(/\.tar@$/, '.tar');
            }));
          }
        });
        that.packagingPlan = {
          prepack: prepack,
          direct: direct
        };
        done();
      });
    }
  }, {
    key: "prepackage",
    value: function prepackage(done) {
      var that = this;
      var tasks = this.packagingPlan.prepack.map(function (instruction) {
        return function (callback) {
          if (instruction.isPackage) {
            // handle additional packages
            that.prepackAdditionalPackages(instruction, callback);
          } else {
            var dir = instruction.paths[0];

            _tar["default"].c({
              file: dir + '.tar',
              cwd: dir,
              portable: true,
              filter: function filter(filePath, stat) {
                var file = filePath.replace(process.cwd() + _path["default"].sep, '').replace(/\\/g, '/');
                return dir == file || !that.isIntermediateFile(file);
              }
            }, _fs["default"].readdirSync(dir), function (err) {
              callback(err);
            });
          }
        };
      });

      _async["default"].waterfall(tasks, function (err) {
        return done(err);
      });
    }
  }, {
    key: "prepackAdditionalPackages",
    value: function prepackAdditionalPackages(instruction, callback) {
      var _this2 = this;

      var dir = instruction.paths[0];
      var options = {
        destination: instruction.original,
        quiet: true
      };

      var runPackager = function runPackager() {
        new _TaskRunner["default"](options).run().then(function (result) {
          return callback();
        })["catch"](callback);
      };

      _fs["default"].stat(dir, function (err, stats) {
        if (!err) {
          options.source = dir;
          runPackager();
        } // try to find package directory
        else {
          dir = _path["default"].dirname(dir);

          _this2.findAdditionalPackage(dir, instruction.identifier, function (err, result) {
            if (err) {
              callback(err);
              return;
            }

            if (!result) {
              callback("Unable to locate package '".concat(instruction.identifier, "', which is defined to be included"));
              return;
            }

            options.source = result;
            runPackager();
          });
        }
      });
    }
    /**
     * Tries to find a package by reading all package.xml files
     * in the directory and comparing the identifier name
     * 
     * @param {string} directory directory to recursively look in
     * @param {string} identifier package identifier name to look for
     * @param {function} done callback 
     */

  }, {
    key: "findAdditionalPackage",
    value: function findAdditionalPackage(directory, identifier, done) {
      (0, _glob["default"])(_path["default"].join(directory, '/**/package.xml'), function (err, files) {
        if (err) {
          done(err);
          return;
        }

        if (files.length <= 0) {
          done();
          return;
        }

        var parser = new _PackageXmlParser["default"]();
        var tasks = files.map(function (file) {
          return function (callback) {
            _async["default"].series([function (cb) {
              return parser.readXml(file, cb);
            }, function (cb) {
              return parser.parseInfo(cb);
            }], function (err) {
              if (err) {
                callback(err);
                return;
              }

              var dirFound = null;

              if (parser.info.name === identifier) {
                dirFound = _path["default"].dirname(file);
              }

              callback(null, dirFound);
            });
          };
        }); // run all tasks and only return an error if all failed

        _async["default"].race(_async["default"].reflectAll(tasks), function (err, result) {
          var error = err;

          if (!result.error) {
            if (result.value) {
              done(null, result.value);
              return;
            }
          } else {
            error = result.error;
          }

          done(error);
        });
      });
    }
  }, {
    key: "isIntermediateFile",
    value: function isIntermediateFile(name, omitTar) {
      for (var file in this.filesToPackage) {
        var filename = this.filesToPackage[file].path;

        if (filename == name + '.tar' || omitTar && filename == name) {
          return this.filesToPackage[file].intermediate;
        }
      }

      return false;
    }
  }, {
    key: "packageAll",
    value: function packageAll(done) {
      var that = this;
      var streams = [];
      var files = this.packagingPlan.direct.concat(this.packagingPlan.prepack.map(function (item) {
        return item.original;
      })).map(_path["default"].normalize); // Windows compatibility

      var folders = [];
      files.forEach(function (dir) {
        // Don't include folders that only contain intermediate files
        if (that.isIntermediateFile(dir, true)) return;

        var base = _path["default"].dirname(dir);

        var dirs = [base];

        while (_path["default"].dirname(base) != '.') {
          base = _path["default"].dirname(base);
          dirs.push(base);
        }

        folders = folders.concat(dirs);
      });
      folders = folders.filter(function (el, i, arr) {
        return arr.indexOf(el) === i;
      }); // Make sure directory exists

      var destination = that.getDestinationPath();

      var dir = _path["default"].dirname(destination);

      if (!_fs["default"].existsSync(dir)) {
        _fs["default"].mkdirSync(dir, {
          recursive: true
        });
      }

      _tar["default"].c({
        file: destination,
        portable: true,
        preservePaths: false,
        gzip: destination.substr(-6) === 'tar.gz',
        filter: function filter(filePath, stat) {
          // Remove path up to cwd
          var file = _path["default"].relative(process.cwd(), filePath);

          return !that.isIntermediateFile(file, true) && (!file // Zero-length-string = cwd
          || folders.indexOf(file) !== -1 || files.indexOf(file) !== -1);
        }
      }, _fs["default"].readdirSync(process.cwd()), function (err) {
        done(err, destination);
      });
    }
  }, {
    key: "getDestinationPath",
    value: function getDestinationPath() {
      var destination = this.destination;
      if (!_Util["default"].isTarball(destination)) destination = _path["default"].join(destination, '{name}_v{version}.tar.gz');
      destination = _path["default"].normalize(destination.replace('{name}', this.packageInfo.name).replace('{version}', this.packageInfo.version.replace(/\s+/gi, '_')));
      return destination;
    }
  }, {
    key: "cleanup",
    value: function cleanup(done) {
      if (!this.packagingPlan || !this.packagingPlan.prepack || this.packagingPlan.prepack.length <= 0) {
        done();
        return;
      }

      var deleteTasks = this.packagingPlan.prepack.map(function (it) {
        // make sure it only deletes tarball's
        if (!_Util["default"].isTarball(it.original)) {
          return;
        }

        return function (callback) {
          return _fs["default"].unlink(it.original, function () {
            return callback();
          });
        };
      });

      _async["default"].parallel(deleteTasks, function () {
        return done();
      });
    }
  }, {
    key: "writeTreeStructure",
    value: function writeTreeStructure(quiet, done) {
      if (quiet) return done();
      var that = this;
      var tree = {};
      this.packagingPlan.direct.forEach(function (file) {
        tree = (0, _TreeBuilder.buildTree)(tree, file);
      });
      var nonIntermediatePrepacks = this.packagingPlan.prepack.filter(function (it) {
        return !that.isIntermediateFile(it.paths[0]);
      });
      nonIntermediatePrepacks.forEach(function (it) {
        tree = (0, _TreeBuilder.buildTree)(tree, it.original);
      });
      console.log(_chalk["default"].bold.green(_path["default"].basename(this.getDestinationPath())));
      (0, _TreeBuilder.outputTree)(tree);
      done();
    }
  }]);
  return Packager;
}();

exports["default"] = Packager;