"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _fs = _interopRequireDefault(require("fs"));

var _lodash = _interopRequireDefault(require("lodash.defaults"));

var _path = _interopRequireDefault(require("path"));

var _process = _interopRequireDefault(require("process"));

var _async = _interopRequireDefault(require("async"));

var _Packager = _interopRequireDefault(require("./Packager"));

var _PackageXmlParser = _interopRequireDefault(require("./PackageXmlParser"));

var TaskRunner = /*#__PURE__*/function () {
  function TaskRunner(options) {
    (0, _classCallCheck2["default"])(this, TaskRunner);
    this.options = (0, _lodash["default"])(options, {
      pips: {},
      quiet: false,
      source: '.',
      destination: '.',
      cwd: _process["default"].cwd()
    }); // resolve paths into absolute paths

    this.options.source = _path["default"].resolve(this.options.cwd, this.options.source);
    this.options.destination = _path["default"].resolve(this.options.cwd, this.options.destination);
    this.xml = null;
  }

  (0, _createClass2["default"])(TaskRunner, [{
    key: "run",
    value: function run() {
      var _this = this;

      var that = this; // TODO don't use chdir to prevent potential issues with cwd unexpectedly changing

      this.oldCwd = _process["default"].cwd();

      _process["default"].chdir(this.options.source);

      return new Promise(function (resolve, reject) {
        // Run the following instructions in series.
        _async["default"].series([function (cb) {
          return that.doesPackageXmlExist(cb);
        }, function (cb) {
          return that.readAndParseFile(cb);
        }, function (cb) {
          return that.runPackager(cb);
        }], function (err, results) {
          // restore current working directory
          _process["default"].chdir(_this.oldCwd); // If any of the functions threw an error, exit here.


          if (err) {
            reject(err);
          } else {
            resolve(results.pop());
          }
        });
      });
    }
  }, {
    key: "doesPackageXmlExist",
    value: function doesPackageXmlExist(callback) {
      _fs["default"].stat('package.xml', function (err, stats) {
        if (err) {
          return callback(new Error("The package.xml could not be accessed in this folder."));
        }

        callback(null, true);
      });
    }
  }, {
    key: "readAndParseFile",
    value: function readAndParseFile(callback) {
      var parser = new _PackageXmlParser["default"]();
      parser.parse(this, callback);
    }
  }, {
    key: "runPackager",
    value: function runPackager(callback) {
      var packager = new _Packager["default"](this.filesToPackage, this.xmlInfo);
      packager.run(callback, this.options.destination, this.options.quiet);
    }
  }]);
  return TaskRunner;
}();

exports["default"] = TaskRunner;